@@ -18,6 +18,7 @@
 
 /* Tests reading ISO 9660 info from an ISO 9660 image.  */
 #include "portable.h"
+#include "cdio_assert.h"
 
 #ifdef HAVE_CONFIG_H
 # include "config.h"
@@ -77,6 +78,7 @@ main(int argc, const char *argv[])
     }
 
 #ifdef HAVE_JOLIET
+    iso9660_close(p_iso);
     p_iso = iso9660_open_ext(ISO9660_IMAGE, ISO_EXTENSION_ALL);
     joliet_level = iso9660_ifs_get_joliet_level(p_iso);
     if ( joliet_level != 3) {
@@ -105,9 +107,11 @@ main(int argc, const char *argv[])
       char buf[ISO_BLOCKSIZE];
       char *psz_path = NULL;
       const lsn_t i_lsn = p_statbuf->lsn;
-      const iso9660_stat_t *p_statbuf2 = iso9660_ifs_find_lsn (p_iso, i_lsn);
-      const iso9660_stat_t *p_statbuf3 =
+      iso9660_stat_t *p_statbuf2 = iso9660_ifs_find_lsn (p_iso, i_lsn);
+      iso9660_stat_t *p_statbuf3 =
 	iso9660_ifs_find_lsn_with_path (p_iso, i_lsn, &psz_path);
+      int rc=0;
+      const unsigned int statbuf_test_size = 100;
 
       /* Compare the two statbufs. */
       if (p_statbuf->lsn != p_statbuf2->lsn ||
@@ -116,25 +120,36 @@ main(int argc, const char *argv[])
 
 	  fprintf(stderr, "File stat information between fs_stat and "
 		  "iso9660_ifs_find_lsn isn't the same\n");
-	  exit(3);
+	  printf("statbuf  lsn: %d, size: %d, type: %d\n",
+		 p_statbuf->lsn, p_statbuf->size, p_statbuf->type);
+	  printf("statbuf2 lsn: %d, size: %d, type: %d\n",
+		 p_statbuf2->lsn, p_statbuf2->size, p_statbuf2->type);
+	  rc=3;
+	  goto exit;
       }
 
-      if (p_statbuf3->lsn != p_statbuf2->lsn ||
-	  p_statbuf3->size != p_statbuf2->size ||
-	  p_statbuf3->type != p_statbuf2->type) {
-	  exit(4);
+      cdio_assert(statbuf_test_size < sizeof(iso9660_stat_t));
+      if (0 != memcmp(p_statbuf3, p_statbuf2, statbuf_test_size)) {
+	  fprintf(stderr, "File stat information between fs_find_lsn and "
+		  "fs_find_lsn_with_path isn't the same\n");
+	  rc=4;
+	  goto exit;
       }
 
       if (psz_path != NULL) {
 	if (0 != strncmp("/./", psz_path, strlen("/./"))) {
 	  fprintf(stderr, "Path returned for ifs_find_lsn_with_path "
 		  "is not correct should be /./, is %s\n", psz_path);
-	  exit(5);
+	  free(psz_path);
+	  rc=5;
+	  goto exit;
 	}
 	free(psz_path);
       } else {
 	fprintf(stderr, "Path returned for fs_find_lsn_with_path is NULL\n");
-	exit(6);
+	free(psz_path);
+	rc=6;
+	goto exit;
       }
 
       /* Try reading from the directory. */
@@ -143,9 +158,14 @@ main(int argc, const char *argv[])
 	{
 	  fprintf(stderr, "Error reading ISO 9660 file at lsn %lu\n",
 		  (long unsigned int) p_statbuf->lsn);
-	  exit(7);
+	  rc=7;
 	}
-      exit(0);
+    exit:
+      iso9660_stat_free(p_statbuf);
+      iso9660_stat_free(p_statbuf2);
+      iso9660_stat_free(p_statbuf3);
+      iso9660_close(p_iso);
+      exit(rc);
     }
   }
 
